<?php

namespace Drupal\streaming_media_views\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\file\Entity\File;
use Drupal\media\Entity\Media;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Manage video content.
 */
class ContentController extends ControllerBase {

  /**
   * Transcode action.
   *
   * @param string $mid
   *   Media id.
   *
   * @return array
   *   Response data to template.
   */
  public function transcode(string $mid): array {

    $result = \Drupal::service('streaming.transcode')->video(
      $this->getParameters($mid)
    );

    if ($result['status'] === TRUE) {
      $extra = [
        'mid' => $mid,
        'playlist' => $result['playlist']['url']['playlist_real_url'],
        'poster' => $this->preparePoster($result),
      ];

      // Setting poster to video preview.
      // I try to set the poster of `Media: streaming_video`,
      // otherwise I try to use the first generated thumbnails,
      // or as a last option I put FALSE and the player will use
      // the first frame of the video as poster.
      // Read from the bottom up.
      $result['media-poster-preview'] =
        // First thumbnail generated.
        $this->getMediaPoster($mid)['url'] === FALSE && isset($extra['poster'])
        ? \Drupal::service('stream_wrapper_manager')
          ->getViaUri(array_key_first($extra['poster']))->getExternalUrl()
        // Otherwise, media->field_poster['url'] or FALSE.
        : $this->getMediaPoster($mid)['url'];

      $form = \Drupal::formBuilder()->getForm('Drupal\streaming_media_views\Form\TranscodeForm', $extra);
    }
    else {
      $form = FALSE;
      \Drupal::messenger()->addError($this->t("Failed video transcode. See the log for details."));
    }

    return [
      '#theme' => 'streaming-media-views-transcode',
      '#result' => $result,
      '#form' => $form,
    ];
  }

  /**
   * Refresh video streaming.
   *
   * @param string $mid
   *   Media id.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Response data to template.
   */
  public function refresh(string $mid): RedirectResponse {
    $videoParameters = $this->getParameters($mid);
    $videoParameters['refresh-if-exists'] = TRUE;

    $result = \Drupal::service('streaming.transcode')->video($videoParameters);

    if ($result['status'] === TRUE) {
      \Drupal::messenger()->addMessage($this->t("Refresh success."));
    }
    else {
      \Drupal::messenger()->addError($this->t("Failed video transcode. See the log for details."));
    }

    return $this->redirect('streaming.transcode', ['mid' => $mid]);
  }

  /**
   * Get video parameters from Media.
   *
   * @param int $mid
   *   Media id.
   *
   * @return array
   *   Video data configuration.
   */
  private function getParameters(int $mid): array {
    $mediaObject = Media::load($mid);
    $file = File::load($mediaObject->field_media_video_file->target_id);

    return [
      'video-data' => [
        'uri'  => \Drupal::service('file_system')->realpath($file->getFileUri()),
        'name' => $file->getFilename(),
        'id'   => $file->id(),
        'uuid' => $file->uuid(),
      ],
    ];
  }

  /**
   * Prepare poster radio options.
   *
   * @param array $result
   *   Data width jpg and gif array.
   *
   * @return array|null
   *   If there are data, it returns the processed data.
   */
  private function preparePoster(array $result): ?array {
    $poster = [];

    foreach ($result['thumbnails']['jpg'] as $key => $item) {
      $poster[$item['relative_path']] = '<img src="' . $item['url'] . '" width="200" height="200" />';
    }
    foreach ($result['thumbnails']['gif'] as $key => $item) {
      $poster[$item['relative_path']] = '<img src="' . $item['url'] . '" width="200" height="200" />';
    }

    return count($poster) > 0 ? $poster : NULL;
  }

  /**
   * Return media poster if it is set.
   *
   * @param int $mid
   *   Media id.
   *
   * @return array
   *   Result array [ 'url' => string , 'uri' => string ]
   */
  private function getMediaPoster(int $mid): array {
    $mediaObject = Media::load($mid);
    $file = $mediaObject->field_poster->target_id
      ? File::load($mediaObject->field_poster->target_id)
      : FALSE;

    if ($file) {
      return [
        'url' => \Drupal::service('stream_wrapper_manager')
          ->getViaUri($file->getFileUri())->getExternalUrl(),
        'uri' => $file->getFileUri(),
      ];
    }
    else {
      return ['url' => FALSE, 'uri' => FALSE];
    }

  }

}
